/**
 * Returns accuracy.
 * @param {*[]} pred Predicted classes
 * @param {*[]} t True classes
 * @returns {number} Accuracy
 */
export function accuracy(pred: any[], t: any[]): number;
/**
 * Returns precision with macro average.
 * @param {*[]} pred Predicted classes
 * @param {*[]} t True classes
 * @returns {number} Precision
 */
export function precision(pred: any[], t: any[]): number;
/**
 * Returns recall with macro average.
 * @param {*[]} pred Predicted classes
 * @param {*[]} t True classes
 * @returns {number} Recall
 */
export function recall(pred: any[], t: any[]): number;
/**
 * Returns F-score with macro average.
 * @param {*[]} pred Predicted classes
 * @param {*[]} t True classes
 * @param {number} [beta] Positive real factor. Recall is considered `beta` times as important as precision.
 * @returns {number} F-score
 */
export function fScore(pred: any[], t: any[], beta?: number): number;
/**
 * Returns Cohen's kappa coefficient.
 * @param {*[]} pred Predicted classes
 * @param {*[]} t True classes
 * @returns {number} Cohen's kappa coefficient
 */
export function cohensKappa(pred: any[], t: any[]): number;
