/**
 * Returns Davies-Bouldin index.
 * @param {Array<Array<number>>} data Original data
 * @param {*[]} pred Predicted categories
 * @param {number} p P
 * @param {number} q Q
 * @returns {number} Davies-Bouldin index
 */
export function davisBouldinIndex(data: Array<Array<number>>, pred: any[], p?: number, q?: number): number;
/**
 * Returns Silhouette coefficient.
 * @param {Array<Array<number>>} data Original data
 * @param {*[]} pred Predicted categories
 * @returns {number[]} Silhouette coefficient
 */
export function silhouetteCoefficient(data: Array<Array<number>>, pred: any[]): number[];
/**
 * Returns Dunn index.
 * @param {Array<Array<number>>} data Original data
 * @param {*[]} pred Predicted categories
 * @param {'max' | 'mean' | 'centroid'} intra_d Intra-cluster distance type
 * @param {'centroid'} inter_d Inter-cluster distance type
 * @returns {number} Dunn index
 */
export function dunnIndex(data: Array<Array<number>>, pred: any[], intra_d?: 'max' | 'mean' | 'centroid', inter_d?: 'centroid'): number;
/**
 * Returns Purity.
 * @param {*[]} pred Predicted categories
 * @param {*[]} t True categories
 * @returns {number} Purity
 */
export function purity(pred: any[], t: any[]): number;
/**
 * Returns Rand index.
 * @param {*[]} pred Predicted categories
 * @param {*[]} t True categories
 * @returns {number} Rank index
 */
export function randIndex(pred: any[], t: any[]): number;
/**
 * Returns Dice index.
 * @param {*[]} pred Predicted categories
 * @param {*[]} t True categories
 * @param {number} [beta] Positive real factor. Recall is considered `beta` times as important as precision.
 * @returns {number} Dice index
 */
export function diceIndex(pred: any[], t: any[], beta?: number): number;
/**
 * Returns Jaccard index.
 * @param {*[]} pred Predicted categories
 * @param {*[]} t True categories
 * @returns {number} Jaccard index
 */
export function jaccardIndex(pred: any[], t: any[]): number;
/**
 * Returns Fowlkes-Mallows index.
 * @param {*[]} pred Predicted categories
 * @param {*[]} t True categories
 * @returns {number} Fowlkes-Mallows index
 */
export function fowlkesMallowsIndex(pred: any[], t: any[]): number;
