/**
 * Returns MSE (Mean Squared Error).
 * @param {number[] | Array<Array<number>>} pred Predicted values
 * @param {number[] | Array<Array<number>>} t True values
 * @returns {number | number[]} Mean Squared Error
 */
export function mse(pred: number[] | Array<Array<number>>, t: number[] | Array<Array<number>>): number | number[];
/**
 * Returns RMSE (Root Mean Squared Error).
 * @param {number[] | Array<Array<number>>} pred Predicted values
 * @param {number[] | Array<Array<number>>} t True values
 * @returns {number | number[]} Root Mean Squared Error
 */
export function rmse(pred: number[] | Array<Array<number>>, t: number[] | Array<Array<number>>): number | number[];
/**
 * Returns MAE (Mean Absolute Error).
 * @param {number[] | Array<Array<number>>} pred Predicted values
 * @param {number[] | Array<Array<number>>} t True values
 * @returns {number | number[]} Mean Absolute Error
 */
export function mae(pred: number[] | Array<Array<number>>, t: number[] | Array<Array<number>>): number | number[];
/**
 * Returns MAD (Median Absolute Deviation).
 * @param {number[] | Array<Array<number>>} pred Predicted values
 * @param {number[] | Array<Array<number>>} t True values
 * @returns {number | number[]} Median Absolute Deviation
 */
export function mad(pred: number[] | Array<Array<number>>, t: number[] | Array<Array<number>>): number | number[];
/**
 * Returns RMSPE (Root Mean Squared Percentage Error).
 * @param {number[] | Array<Array<number>>} pred Predicted values
 * @param {number[] | Array<Array<number>>} t True values
 * @returns {number | number[]} Root Mean Squared Percentage Error
 */
export function rmspe(pred: number[] | Array<Array<number>>, t: number[] | Array<Array<number>>): number | number[];
/**
 * Returns MAPE (Mean Absolute Percentage Error).
 * @param {number[] | Array<Array<number>>} pred Predicted values
 * @param {number[] | Array<Array<number>>} t True values
 * @returns {number | number[]} Mean Absolute Percentage Error
 */
export function mape(pred: number[] | Array<Array<number>>, t: number[] | Array<Array<number>>): number | number[];
/**
 * Returns MSLE (Mean Squared Logarithmic Error).
 * @param {number[] | Array<Array<number>>} pred Predicted values
 * @param {number[] | Array<Array<number>>} t True values
 * @returns {number | number[]} Mean Squared Logarithmic Error
 */
export function msle(pred: number[] | Array<Array<number>>, t: number[] | Array<Array<number>>): number | number[];
/**
 * Returns RMSLE (Root Mean Squared Logarithmic Error).
 * @param {number[] | Array<Array<number>>} pred Predicted values
 * @param {number[] | Array<Array<number>>} t True values
 * @returns {number | number[]} RootMean Squared Logarithmic Error
 */
export function rmsle(pred: number[] | Array<Array<number>>, t: number[] | Array<Array<number>>): number | number[];
/**
 * Returns R2 (coefficient of determination).
 * @param {number[] | Array<Array<number>>} pred Predicted values
 * @param {number[] | Array<Array<number>>} t True values
 * @returns {number | number[]} Coefficient of determination
 */
export function r2(pred: number[] | Array<Array<number>>, t: number[] | Array<Array<number>>): number | number[];
/**
 * Returns correlation.
 * @function
 * @param {number[] | Array<Array<number>>} pred Predicted values
 * @param {number[] | Array<Array<number>>} t True values
 * @returns {number | number[]} Correlation
 */
export function correlation(pred: number[] | Array<Array<number>>, t: number[] | Array<Array<number>>): number | number[];
