/**
 * Approximate Large Margin algorithm
 * @see A New Approximate Maximal Margin Classification Algorithm. (2001)
 */
export default class ALMA {
    /**
     * @param {number} p Power parameter for norm
     * @param {number} alpha Degree of approximation to the optimal margin hyperplane
     * @param {number} b Tuning parameter
     * @param {number} c Tuning parameter
     */
    constructor(p?: number, alpha?: number, b?: number, c?: number);
    _p: number;
    _alpha: number;
    _b: number;
    _c: number;
    _w: any;
    _w0: number;
    _k: number;
    /**
     * Update model parameters with one data.
     * @param {number[]} x Training data
     * @param {1 | -1} y Target value
     */
    update(x: number[], y: 1 | -1): void;
    /**
     * Fit model parameters.
     * @param {Array<Array<number>>} x Training data
     * @param {Array<1 | -1>} y Target values
     */
    fit(x: Array<Array<number>>, y: Array<1 | -1>): void;
    /**
     * Returns predicted datas.
     * @param {Array<Array<number>>} data Sample data
     * @returns {(1 | -1)[]} Predicted values
     */
    predict(data: Array<Array<number>>): (1 | -1)[];
}
