/**
 * Autoregressive model
 */
export default class AR {
    /**
     * @param {number} p Order
     * @param {'lsm' | 'yuleWalker' | 'levinson' | 'householder'} [method] Method name
     */
    constructor(p: number, method?: 'lsm' | 'yuleWalker' | 'levinson' | 'householder');
    _p: number;
    _method: "lsm" | "yuleWalker" | "levinson" | "householder";
    /**
     * Fit model.
     * @param {number[]} data Training data
     */
    fit(data: number[]): void;
    _lsm(x: any): void;
    _phi: Matrix<number> | Matrix<any>;
    _yuleWalker(x: any): void;
    _variance: number;
    _levinson(x: any): void;
    _householder(x: any): void;
    /**
     * Returns predicted future values.
     * @param {number[]} data Sample data
     * @param {number} k Prediction count
     * @returns {number[]} Predicted values
     */
    predict(data: number[], k: number): number[];
}
import Matrix from '../util/matrix.js';
