/**
 * Association analysis
 */
export default class AssociationAnalysis {
    /**
     * @param {number} support Minimum support
     */
    constructor(support: number);
    _support: number;
    /**
     * Fit model.
     * @param {Array<Array<*>>} x Training data
     */
    fit(x: Array<Array<any>>): void;
    _x: any[][];
    _commons: SetKeyMap[];
    /**
     * Returns appearing keys.
     * @param {number} n Length of key
     * @returns {Iterator<string[]>} Appearing keys
     */
    items(n?: number): Iterator<string[]>;
    /**
     * Returns support value.
     * @param {...*} a Keys
     * @returns {number} Support value
     */
    support(...a: any[]): number;
    /**
     * Returns confidence value.
     * @param {*} a Key
     * @param {*} b Key
     * @returns {number} Confidence value
     */
    confidence(a: any, b: any): number;
    /**
     * Returns lift value.
     * @param {*} a Key
     * @param {*} b Key
     * @returns {number} Lift value
     */
    lift(a: any, b: any): number;
}
declare class SetKeyMap {
    _map: Map<any, any>;
    _keymap: {};
    get size(): number;
    _getsamekey(key: any): any;
    clear(): void;
    delete(key: any): boolean;
    get(key: any): any;
    has(key: any): boolean;
    set(key: any, value: any): void;
    keys(): IterableIterator<any>;
    values(): IterableIterator<any>;
    entries(): IterableIterator<[any, any]>;
    forEach(callback: any, thisArg: any): void;
    [Symbol.iterator](): Generator<any, void, undefined>;
}
export {};
