/**
 * @ignore
 * @typedef {import("./nns/graph").LayerObject} LayerObject
 */
/**
 * Autoencoder
 */
export default class Autoencoder {
    /**
     * @param {number} input_size Input size
     * @param {number} reduce_size Reduced dimension
     * @param {LayerObject[]} enc_layers Layers of encoder
     * @param {LayerObject[]} dec_layers Layers of decoder
     * @param {string} optimizer Optimizer of the network
     */
    constructor(input_size: number, reduce_size: number, enc_layers: LayerObject[], dec_layers: LayerObject[], optimizer: string);
    _input_size: number;
    _layers: {
        type: string;
        name: string;
    }[];
    _model: NeuralNetwork;
    _epoch: number;
    /**
     * Epoch
     * @type {number}
     */
    get epoch(): number;
    /**
     * Fit model.
     * @param {Array<Array<number>>} train_x Training data
     * @param {number} iteration Iteration count
     * @param {number} rate Learning rate
     * @param {number} batch Batch size
     * @param {number} rho Sparsity parameter
     * @returns {number} Loss value
     */
    fit(train_x: Array<Array<number>>, iteration: number, rate: number, batch: number, rho: number): number;
    /**
     * Returns predicted datas.
     * @param {Array<Array<number>>} x Sample data
     * @returns {Array<Array<number>>} Predicted values
     */
    predict(x: Array<Array<number>>): Array<Array<number>>;
    /**
     * Returns reduced datas.
     * @param {Array<Array<number>>} x Sample data
     * @returns {Array<Array<number>>} Predicted values
     */
    reduce(x: Array<Array<number>>): Array<Array<number>>;
}
export type LayerObject = import("./nns/graph").LayerObject;
import NeuralNetwork from './neuralnetwork.js';
