/**
 * Bounded Online Gradient Descent
 */
export default class BOGD {
    /**
     * @param {number} [b] Maximum budget size
     * @param {number} [eta] Stepsize
     * @param {number} [lambda] Regularization parameter
     * @param {number} [gamma] Maximum coefficient
     * @param {'uniform' | 'nonuniform'} [sampling] Sampling approach
     * @param {'gaussian' | 'polynomial' | { name: 'gaussian', s?: number } | { name: 'polynomial', d?: number } | function (number[], number[]): number} [kernel] Kernel name
     * @param {'zero_one' | 'hinge'} [loss] Loss type name
     */
    constructor(b?: number, eta?: number, lambda?: number, gamma?: number, sampling?: 'uniform' | 'nonuniform', kernel?: 'gaussian' | 'polynomial' | {
        name: 'gaussian';
        s?: number;
    } | {
        name: 'polynomial';
        d?: number;
    } | ((arg0: number[], arg1: number[]) => number), loss?: 'zero_one' | 'hinge');
    _b: number;
    _eta: number;
    _lambda: number;
    _gamma: number;
    _sampling: "uniform" | "nonuniform";
    _kernel: any;
    _gloss: (t: any, y: any) => 0 | -1;
    _sv: any[];
    _alpha: any[];
    /**
     * Fit model.
     * @param {Array<Array<number>>} x Training data
     * @param {Array<1 | -1>} y Target values
     */
    fit(x: Array<Array<number>>, y: Array<1 | -1>): void;
    /**
     * Returns predicted values.
     * @param {Array<Array<number>>} data Sample data
     * @returns {(1 | -1)[]} Predicted values
     */
    predict(data: Array<Array<number>>): (1 | -1)[];
}
