/**
 * Budgeted online Passive-Aggressive
 */
export default class BPA {
    /**
     * @param {number} [c] Regularization parameter
     * @param {number} [b] Budget size
     * @param {'simple' | 'projecting' | 'nn'} [version] Version
     * @param {'gaussian' | 'polynomial' | { name: 'gaussian', s?: number } | { name: 'polynomial', d?: number } | function (number[], number[]): number} [kernel] Kernel name
     */
    constructor(c?: number, b?: number, version?: 'simple' | 'projecting' | 'nn', kernel?: 'gaussian' | 'polynomial' | {
        name: 'gaussian';
        s?: number;
    } | {
        name: 'polynomial';
        d?: number;
    } | ((arg0: number[], arg1: number[]) => number));
    _c: number;
    _b: number;
    _version: "simple" | "projecting" | "nn";
    _kernel: any;
    _sv: any[];
    _nn: number;
    /**
     * Fit model.
     * @param {Array<Array<number>>} x Training data
     * @param {Array<1 | -1>} y Target values
     */
    fit(x: Array<Array<number>>, y: Array<1 | -1>): void;
    /**
     * Returns predicted values.
     * @param {Array<Array<number>>} data Sample data
     * @returns {(1 | -1)[]} Predicted values
     */
    predict(data: Array<Array<number>>): (1 | -1)[];
}
