/**
 * Clustering based on Closest Pairs
 */
export default class C2P {
    /**
     * @param {number} r Number of representative points
     * @param {number} m Number of required sub-clusters
     */
    constructor(r: number, m: number);
    _r: number;
    _m: number;
    _cutoff_scale: number;
    _distance(a: any, b: any): number;
    _mean(a: any): any;
    /**
     * Fit model.
     * @param {Array<Array<number>>} data Training data
     */
    fit(data: Array<Array<number>>): void;
    _n: number;
    _root: {
        repr: any;
        index: number[];
        distance: number;
        children: {
            repr: number[][];
            index: number[];
            distance: number;
            children: any[];
        }[];
    };
    /**
     * Returns the specified number of clusters.
     * @param {number} number Number of clusters
     * @returns {{repr: number[][], index: number[], distance: number, children: *[]}[]} Cluster nodes
     */
    getClusters(number: number): {
        repr: number[][];
        index: number[];
        distance: number;
        children: any[];
    }[];
    /**
     * Returns predicted categories.
     * @param {number} k Number of clusters
     * @returns {number[]} Predicted values
     */
    predict(k: number): number[];
}
