/**
 * Catmull-Rom splines interpolation
 */
export class CatmullRomSplines {
    /**
     * Fit model parameters.
     * @param {number[]} x Training data
     * @param {number[]} y Target values
     */
    fit(x: number[], y: number[]): void;
    _x: number[];
    _y: number[];
    /**
     * Returns predicted interpolated values.
     * @param {number[]} target Sample data
     * @returns {number[]} Predicted values
     */
    predict(target: number[]): number[];
}
/**
 * Centripetal Catmull-Rom splines interpolation
 */
export class CentripetalCatmullRomSplines {
    /**
     * @param {number} alpha Number for knot parameterization
     */
    constructor(alpha?: number);
    _alpha: number;
    /**
     * Fit model parameters.
     * @param {number[]} x Training data
     * @param {number[]} y Target values
     */
    fit(x: number[], y: number[]): void;
    _x: number[];
    _y: number[];
    /**
     * Returns predicted interpolated values.
     * @param {number[]} target Sample data
     * @returns {number[]} Predicted values
     */
    predict(target: number[]): number[];
}
