/**
 * Confidence weighted
 */
export class ConfidenceWeighted {
    /**
     * @param {number} eta Confidence value
     */
    constructor(eta: number);
    _eta: number;
    _phi: any;
    _psi: number;
    _xi: number;
    /**
     * Initialize this model.
     * @param {Array<Array<number>>} train_x Training data
     * @param {Array<1 | -1>} train_y Target values
     */
    init(train_x: Array<Array<number>>, train_y: Array<1 | -1>): void;
    _x: Matrix<number[]>;
    _c: Matrix<number>;
    _y: (1 | -1)[];
    _d: number;
    _m: Matrix<number>;
    _s: Matrix<number>;
    _cdf(x: any): number;
    _ppf(x: any): any;
    _alpha(v: any, m: any): number;
    /**
     * Update model parameters with one data.
     * @param {Matrix} x Training data
     * @param {1 | -1} y Target value
     */
    update(x: Matrix, y: 1 | -1): void;
    /**
     * Fit model parameters.
     */
    fit(): void;
    /**
     * Returns predicted datas.
     * @param {Array<Array<number>>} data Sample data
     * @returns {(1 | -1)[]} Predicted values
     */
    predict(data: Array<Array<number>>): (1 | -1)[];
}
/**
 * Soft confidence weighted
 */
export class SoftConfidenceWeighted extends ConfidenceWeighted {
    /**
     * @param {number} eta Confidence value
     * @param {number} cost Tradeoff value between passiveness and aggressiveness
     * @param {1 | 2} v Version number
     */
    constructor(eta: number, cost: number, v: 1 | 2);
    _cost: number;
    _v: 2 | 1;
}
import Matrix from '../util/matrix.js';
