/**
 * Density-based Optimal projective Clustering
 */
export class DOC {
    /**
     * @param {number} alpha Dense scale
     * @param {number} beta Balanced value
     * @param {number} w Width of cluster
     */
    constructor(alpha: number, beta: number, w: number);
    _alpha: number;
    _beta: number;
    _w: number;
    _p: any[];
    _d: any[];
    _mu: (a: any, b: any) => number;
    _select(n: any, k: any): number[];
    /**
     * Fit model.
     * @param {Array<Array<number>>} datas Sample data
     */
    fit(datas: Array<Array<number>>): void;
    /**
     * Returns predicted categories.
     * @returns {number[]} Predicted values
     */
    predict(): number[];
}
/**
 * Fast Density-based Optimal projective Clustering
 */
export class FastDOC {
    /**
     * @param {number} alpha Dense scale
     * @param {number} beta Balanced value
     * @param {number} w Width of cluster
     * @param {number} maxiter Maximum inner iteration
     * @param {number} d0 Threshold of selected dimension count
     */
    constructor(alpha: number, beta: number, w: number, maxiter: number, d0: number);
    _alpha: number;
    _beta: number;
    _w: number;
    _maxiter: number;
    _d0: number;
    _p: any[];
    _d: any[];
    _select(n: any, k: any): number[];
    /**
     * Fit model.
     * @param {Array<Array<number>>} datas Sample data
     */
    fit(datas: Array<Array<number>>): void;
    /**
     * Returns predicted categories.
     * @returns {number[]} Predicted values
     */
    predict(): number[];
}
