/**
 * Extreme learning machine classifier
 */
export class ELMClassifier extends ELM {
    /**
     * @param {number[]} size Size of hidden layer
     * @param {'identity' | 'elu' | 'gaussian' | 'leaky_relu' | 'sigmoid' | 'softplus' | 'softsign' | 'tanh' | function(number): number} [activation] Activation name
     */
    constructor(size: number[], activation?: 'identity' | 'elu' | 'gaussian' | 'leaky_relu' | 'sigmoid' | 'softplus' | 'softsign' | 'tanh' | ((arg0: number) => number));
    _classes: any[];
    /**
     * Category list
     * @type {*[]}
     */
    get categories(): any[];
    /**
     * Fit model.
     * @param {Array<Array<number>>} x Training data
     * @param {*[]} y Target values
     */
    fit(x: Array<Array<number>>, y: any[]): void;
    /**
     * Returns predicted probabilities.
     * @param {Array<Array<number>>} x Sample data
     * @returns {Array<Array<number>>} Predicted values
     */
    probability(x: Array<Array<number>>): Array<Array<number>>;
    /**
     * Returns predicted values.
     * @param {Array<Array<number>>} x Sample data
     * @returns {*[]} Predicted values
     */
    predict(x: Array<Array<number>>): any[];
}
/**
 * Extreme learning machine regressor
 */
export class ELMRegressor extends ELM {
    /**
     * @param {number[]} size Size of hidden layer
     * @param {'identity' | 'elu' | 'gaussian' | 'leaky_relu' | 'sigmoid' | 'softplus' | 'softsign' | 'tanh' | function(number): number} [activation] Activation name
     */
    constructor(size: number[], activation?: 'identity' | 'elu' | 'gaussian' | 'leaky_relu' | 'sigmoid' | 'softplus' | 'softsign' | 'tanh' | ((arg0: number) => number));
    /**
     * Fit model.
     * @param {Array<Array<number>>} x Training data
     * @param {Array<Array<number>>} y Target values
     */
    fit(x: Array<Array<number>>, y: Array<Array<number>>): void;
    /**
     * Returns predicted values.
     * @param {Array<Array<number>>} x Sample data
     * @returns {Array<Array<number>>} Predicted values
     */
    predict(x: Array<Array<number>>): Array<Array<number>>;
}
export type LayerObject = import("./nns/graph").LayerObject;
declare class ELM {
    constructor(size: any, activation: any);
    _size: any;
    _activation: any;
    _a: any;
    _w: Matrix<number>;
    _b: Matrix<number>;
    fit(x: any, y: any): void;
    _beta: any;
    predict(x: any): any;
}
import Matrix from '../util/matrix.js';
export {};
