/**
 * Hierarchical Density-based spatial clustering of applications with noise
 */
export default class HDBSCAN {
    /**
     * @param {number} [minClusterSize] Minimum number of clusters to be recognized as a cluster
     * @param {number} [minPts] Number of neighborhood with core distance
     * @param {'euclid' | 'manhattan' | 'chebyshev' | function (number[], number[]): number} [metric] Metric name
     */
    constructor(minClusterSize?: number, minPts?: number, metric?: 'euclid' | 'manhattan' | 'chebyshev' | ((arg0: number[], arg1: number[]) => number));
    _minClusterSize: number;
    _minPts: number;
    _metric: "euclid" | "manhattan" | "chebyshev" | ((arg0: number[], arg1: number[]) => number);
    _d: (a: any, b: any) => any;
    /**
     * Number of clusters of last predicted
     * @type {number}
     */
    get size(): number;
    /**
     * Returns predicted categories.
     * @param {Array<Array<number>>} datas Training data
     * @returns {number[]} Predicted values
     */
    predict(datas: Array<Array<number>>): number[];
    _lastResult: any[];
}
