/**
 * Kernelized Primal Estimated sub-GrAdientSOlver for SVM
 */
export default class KernelizedPegasos {
    /**
     * @param {number} rate Learning rate
     * @param {'gaussian' | 'polynomial' | { name: 'gaussian', s?: number } | { name: 'polynomial', d?: number } | function (number[], number[]): number} [kernel] Kernel name
     */
    constructor(rate: number, kernel?: 'gaussian' | 'polynomial' | {
        name: 'gaussian';
        s?: number;
    } | {
        name: 'polynomial';
        d?: number;
    } | ((arg0: number[], arg1: number[]) => number));
    _r: number;
    _itr: number;
    _kernel: any;
    /**
     * Initialize this model.
     * @param {Array<Array<number>>} train_x Training data
     * @param {Array<1 | -1>} train_y Target values
     */
    init(train_x: Array<Array<number>>, train_y: Array<1 | -1>): void;
    _x: number[][];
    _y: (1 | -1)[];
    _t: number;
    _a: any[];
    _k: any[];
    /**
     * Fit model parameters.
     */
    fit(): void;
    /**
     * Returns predicted values.
     * @param {Array<Array<number>>} data Sample data
     * @returns {(1 | -1)[]} Predicted values
     */
    predict(data: Array<Array<number>>): (1 | -1)[];
}
