/**
 * Kernelized perceptron
 */
export default class KernelizedPerceptron {
    /**
     * @param {number} [rate] Learning rate
     * @param {'gaussian' | 'polynomial' | { name: 'gaussian', s?: number } | { name: 'polynomial', d?: number } | function (number[], number[]): number} [kernel] Kernel name
     */
    constructor(rate?: number, kernel?: 'gaussian' | 'polynomial' | {
        name: 'gaussian';
        s?: number;
    } | {
        name: 'polynomial';
        d?: number;
    } | ((arg0: number[], arg1: number[]) => number));
    _r: number;
    _kernel: any;
    _i: any[];
    /**
     * Fit model.
     * @param {Array<Array<number>>} x Training data
     * @param {Array<1 | -1>} y Target values
     */
    fit(x: Array<Array<number>>, y: Array<1 | -1>): void;
    /**
     * Returns predicted values.
     * @param {Array<Array<number>>} data Sample data
     * @returns {(1 | -1)[]} Predicted values
     */
    predict(data: Array<Array<number>>): (1 | -1)[];
}
