/**
 * Kullback-Leibler importance estimation procedure
 */
export default class KLIEP {
    /**
     * @param {number[]} sigma Sigmas of normal distribution
     * @param {number} fold Number of folds
     * @param {number} kernelNum Number of kernels
     */
    constructor(sigma: number[], fold: number, kernelNum: number);
    _sigma_cand: number[];
    _fold: number;
    _kernelNum: number;
    _kernel_gaussian(x: any, c: any, s: any): Matrix<number[]>;
    _optimize_alpha(a: any, b: any): Matrix<number>;
    /**
     * Fit model.
     * @param {Array<Array<number>>} x1 Numerator data
     * @param {Array<Array<number>>} x2 Denominator data
     */
    fit(x1: Array<Array<number>>, x2: Array<Array<number>>): void;
    _centers: any;
    _sigma: number;
    _kw: Matrix<number>;
    /**
     * Returns estimated values.
     * @param {Array<Array<number>>} x Sample data
     * @returns {number[]} Predicted values
     */
    predict(x: Array<Array<number>>): number[];
}
import Matrix from '../util/matrix.js';
