/**
 * Least-squares density difference
 */
export class LSDD {
    /**
     * @param {number[]} sigma Sigmas of normal distribution
     * @param {number[]} lambda Regularization parameters
     */
    constructor(sigma: number[], lambda: number[]);
    _sigma_cand: number[];
    _lambda_cand: number[];
    _kernel_gaussian(x: any, c: any, s: any): Matrix<number[]>;
    /**
     * Fit model.
     * @param {Array<Array<number>>} x1 Left value
     * @param {Array<Array<number>>} x2 Right value
     */
    fit(x1: Array<Array<number>>, x2: Array<Array<number>>): void;
    _centers: Matrix<any>;
    _sigma: number;
    _lambda: number;
    _kw: Matrix<number>;
    /**
     * Returns estimated values.
     * @param {Array<Array<number>>} x Sample data
     * @returns {number[]} Predicted values
     */
    predict(x: Array<Array<number>>): number[];
}
/**
 * LSDD for change point detection
 */
export class LSDDCPD {
    /**
     * @param {number} w Window size
     * @param {number} [take] Take number
     * @param {number} [lag] Lag
     */
    constructor(w: number, take?: number, lag?: number);
    _window: number;
    _take: number;
    _lag: number;
    /**
     * Returns anomaly degrees.
     * @param {Array<Array<number>>} datas Sample data
     * @returns {number[]} Predicted values
     */
    predict(datas: Array<Array<number>>): number[];
}
import Matrix from '../util/matrix.js';
