/**
 * One-class support vector machine
 */
export default class OCSVM {
    /**
     * @param {number} nu Nu
     * @param {'gaussian' | 'linear' | { name: 'gaussian', d?: number } | { name: 'linear' } | function (number[], number[]): number} kernel Kernel name
     */
    constructor(nu: number, kernel: 'gaussian' | 'linear' | {
        name: 'gaussian';
        d?: number;
    } | {
        name: 'linear';
    } | ((arg0: number[], arg1: number[]) => number));
    _kernel: any;
    _nu: number;
    _eps: number;
    /**
     * Initialize model.
     * @param {Array<Array<number>>} x Training data
     */
    init(x: Array<Array<number>>): void;
    _x: number[][];
    _nl: number;
    _a: any[];
    _b: number;
    _alldata: boolean;
    _k: any[];
    /**
     * Fit model.
     */
    fit(): void;
    _fitOnce(): number;
    _rho: number;
    /**
     * Returns anomaly degrees.
     * @param {Array<Array<number>>} x Sample data
     * @returns {number[]} Predicted values
     */
    predict(x: Array<Array<number>>): number[];
}
