/**
 * Perceptron
 */
export class Perceptron {
    /**
     * @param {number} rate Learning rate
     */
    constructor(rate: number);
    _r: number;
    _a: any[];
    _b: number;
    /**
     * Fit model.
     * @param {Array<Array<number>>} x Training data
     * @param {Array<1 | -1>} y Target values
     */
    fit(x: Array<Array<number>>, y: Array<1 | -1>): void;
    /**
     * Returns predicted values.
     * @param {Array<Array<number>>} data Sample data
     * @returns {(1 | -1)[]} Predicted values
     */
    predict(data: Array<Array<number>>): (1 | -1)[];
}
/**
 * Averaged perceptron
 */
export class AveragedPerceptron {
    /**
     * @param {number} rate Learning rate
     */
    constructor(rate: number);
    _r: number;
    _epoch: number;
    _a: any[];
    _atotal: any[];
    _b: number;
    _btotal: number;
    /**
     * Fit model.
     * @param {Array<Array<number>>} x Training data
     * @param {Array<1 | -1>} y Target values
     */
    fit(x: Array<Array<number>>, y: Array<1 | -1>): void;
    /**
     * Returns predicted values.
     * @param {Array<Array<number>>} data Sample data
     * @returns {(1 | -1)[]} Predicted values
     */
    predict(data: Array<Array<number>>): (1 | -1)[];
}
/**
 * Multiclass perceptron
 */
export class MulticlassPerceptron {
    /**
     * @param {number} rate Learning rate
     */
    constructor(rate: number);
    _r: number;
    _c: any[];
    _epoch: number;
    _a: any[];
    _b: any[];
    /**
     * Fit model.
     * @param {Array<Array<number>>} x Training data
     * @param {*[]} y Target values
     */
    fit(x: Array<Array<number>>, y: any[]): void;
    /**
     * Returns predicted values.
     * @param {Array<Array<number>>} data Sample data
     * @returns {*[]} Predicted values
     */
    predict(data: Array<Array<number>>): any[];
}
