/**
 * radius neighbor
 */
export class RadiusNeighbor extends RadiusNeighborBase {
    /**
     * Add a data.
     * @param {number[]} point Training data
     * @param {*} category Target value
     */
    add(point: number[], category: any): void;
    /**
     * Add datas.
     * @param {Array<Array<number>>} datas Training data
     * @param {*[]} targets Target values
     */
    fit(datas: Array<Array<number>>, targets: any[]): void;
    /**
     * Returns predicted categories.
     * @param {Array<Array<number>>} datas Sample data
     * @returns {*[]} Predicted values
     */
    predict(datas: Array<Array<number>>): any[];
}
/**
 * radius neighbor regression
 */
export class RadiusNeighborRegression extends RadiusNeighborBase {
    /**
     * Add a data.
     * @param {number[]} point Training data
     * @param {number} category Target value
     */
    add(point: number[], category: number): void;
    /**
     * Add datas.
     * @param {Array<Array<number>>} datas Training data
     * @param {number[]} targets Target values
     */
    fit(datas: Array<Array<number>>, targets: number[]): void;
    /**
     * Returns predicted values.
     * @param {Array<Array<number>>} datas Sample data
     * @returns {(number | null)[]} Predicted values
     */
    predict(datas: Array<Array<number>>): (number | null)[];
}
/**
 * Semi-supervised radius neighbor
 */
export class SemiSupervisedRadiusNeighbor extends RadiusNeighborBase {
    _k: number;
    _orgk: number;
    /**
     * Add a data.
     * @param {number[]} point Training data
     * @param {* | null} category Target value
     */
    add(point: number[], category: any | null): void;
    /**
     * Add datas.
     * @param {Array<Array<number>>} datas Training data
     * @param {(* | null)[]} targets Target values
     */
    fit(datas: Array<Array<number>>, targets: (any | null)[]): void;
    /**
     * Returns predicted values.
     * @returns {*[]} Predicted values
     */
    predict(): any[];
}
/**
 * Bsae class for radius neighbor models
 */
declare class RadiusNeighborBase {
    /**
     * @param {number} [r] Radius to determine neighborhood
     * @param {'euclid' | 'manhattan' | 'chebyshev' | 'minkowski' | function (number[], number[]): number} [metric] Metric name
     */
    constructor(r?: number, metric?: 'euclid' | 'manhattan' | 'chebyshev' | 'minkowski' | ((arg0: number[], arg1: number[]) => number));
    _p: any[];
    _c: any[];
    _r: number;
    _metric: "euclid" | "manhattan" | "chebyshev" | "minkowski" | ((arg0: number[], arg1: number[]) => number);
    _d: (a: any, b: any) => any;
    _near_points(data: any): any[];
    /**
     * Add a data.
     * @param {number[]} point Training data
     * @param {*} [category] Target value
     */
    _add(point: number[], category?: any): void;
}
export {};
