/**
 * Radial basis function network
 */
export default class RadialBasisFunctionNetwork {
    /**
     * @param {'linear' | 'gaussian' | 'multiquadric' | 'inverse quadratic' | 'inverse multiquadric' | 'thin plate' | 'bump'} [rbf] RBF name
     * @param {number} [e] Tuning parameter
     * @param {number} [l] Regularization parameter
     */
    constructor(rbf?: 'linear' | 'gaussian' | 'multiquadric' | 'inverse quadratic' | 'inverse multiquadric' | 'thin plate' | 'bump', e?: number, l?: number);
    _f: (x: any, c: any) => number;
    _l: number;
    /**
     * Fit model.
     * @param {Array<Array<number>>} x Training data
     * @param {Array<Array<number>>} y Target values
     */
    fit(x: Array<Array<number>>, y: Array<Array<number>>): void;
    _x: number[][];
    _w: any[] | Matrix<number>;
    /**
     * Returns predicted values.
     * @param {Array<Array<number>>} target Sample data
     * @returns {number[]} Predicted values
     */
    predict(target: Array<Array<number>>): number[];
}
import Matrix from '../util/matrix.js';
