/**
 * Robust Kernel-based Outlier Factor
 */
export default class RKOF {
    /**
     * @param {number} k Number of neighborhoods
     * @param {number} h Smoothing parameter
     * @param {number} alpha Sensitivity parameter
     * @param {'gaussian' | 'epanechnikov' | 'volcano' | { name: 'gaussian' } | { name: 'epanechnikov' } | { name: 'volcano', beta?: number } | function (number[]): number} [kernel] Kernel name
     */
    constructor(k: number, h: number, alpha: number, kernel?: 'gaussian' | 'epanechnikov' | 'volcano' | {
        name: 'gaussian';
    } | {
        name: 'epanechnikov';
    } | {
        name: 'volcano';
        beta?: number;
    } | ((arg0: number[]) => number));
    _k: number;
    _h: number;
    _alpha: number;
    _s: number;
    _kernel: any;
    _distance(a: any, b: any): number;
    /**
     * Returns anomaly degrees.
     * @param {Array<Array<number>>} datas Training data
     * @returns {number[]} Predicted values
     */
    predict(datas: Array<Array<number>>): number[];
}
