/**
 * Selective sampling Perceptron
 */
export class SelectiveSamplingPerceptron {
    /**
     * @param {number} b Smooth parameter
     * @param {number} rate Learning rate
     */
    constructor(b: number, rate: number);
    _b: number;
    _r: number;
    _w: any[];
    _c: number;
    /**
     * Fit model.
     * @param {Array<Array<number>>} x Training data
     * @param {Array<1 | -1>} y Target values
     */
    fit(x: Array<Array<number>>, y: Array<1 | -1>): void;
    /**
     * Returns predicted values.
     * @param {Array<Array<number>>} data Sample data
     * @returns {(1 | -1)[]} Predicted values
     */
    predict(data: Array<Array<number>>): (1 | -1)[];
}
/**
 * Selective sampling Perceptron with adaptive parameter
 */
export class SelectiveSamplingAdaptivePerceptron {
    /**
     * @param {number} beta Smooth parameter
     * @param {number} rate Learning rate
     */
    constructor(beta: number, rate: number);
    _beta: number;
    _r: number;
    _w: any[];
    _c: number;
    _k: number;
    _X: number;
    /**
     * Fit model.
     * @param {Array<Array<number>>} x Training data
     * @param {Array<1 | -1>} y Target values
     */
    fit(x: Array<Array<number>>, y: Array<1 | -1>): void;
    /**
     * Returns predicted values.
     * @param {Array<Array<number>>} data Sample data
     * @returns {(1 | -1)[]} Predicted values
     */
    predict(data: Array<Array<number>>): (1 | -1)[];
}
