/**
 * Relative unconstrained Least-Squares Importance Fitting
 */
export class RuLSIF {
    /**
     * @param {number[]} sigma Sigmas of normal distribution
     * @param {number[]} lambda Regularization parameters
     * @param {number} alpha Relative parameter
     * @param {number} kernelNum Number of kernels
     */
    constructor(sigma: number[], lambda: number[], alpha: number, kernelNum: number);
    _sigma_cand: number[];
    _lambda_cand: number[];
    _alpha: number;
    _kernelNum: number;
    _kernel_gaussian(x: any, c: any, s: any): Matrix<number[]>;
    /**
     * Fit model.
     * @param {Array<Array<number>>} x1 Numerator data
     * @param {Array<Array<number>>} x2 Denominator data
     */
    fit(x1: Array<Array<number>>, x2: Array<Array<number>>): void;
    _centers: any;
    _sigma: number;
    _lambda: number;
    _kw: Matrix<number>;
    /**
     * Returns estimated values.
     * @param {Array<Array<number>>} x Sample data
     * @returns {number[]} Predicted values
     */
    predict(x: Array<Array<number>>): number[];
}
/**
 * unconstrained Least-Squares Importance Fitting
 */
export class uLSIF extends RuLSIF {
    /**
     * @param {number[]} sigma Sigma of normal distribution
     * @param {number[]} lambda Regularization parameters
     * @param {number} kernelNum Number of kernels
     */
    constructor(sigma: number[], lambda: number[], kernelNum: number);
}
import Matrix from '../util/matrix.js';
