/**
 * Weighted k-means model
 */
export default class WeightedKMeans {
    /**
     * @param {number} beta Tuning parameter
     */
    constructor(beta: number);
    _beta: number;
    _centroids: any[];
    _w: any[];
    /**
     * Centroids
     * @type {Array<Array<number>>}
     */
    get centroids(): number[][];
    /**
     * Number of clusters.
     * @type {number}
     */
    get size(): number;
    _distance(a: any, b: any): number;
    /**
     * Add a new cluster.
     * @param {Array<Array<number>>} datas Training data
     * @returns {number[]} Added centroid
     */
    add(datas: Array<Array<number>>): number[];
    /**
     * Clear all clusters.
     */
    clear(): void;
    /**
     * Returns predicted categories.
     * @param {Array<Array<number>>} datas Sample data
     * @returns {number[]} Predicted values
     */
    predict(datas: Array<Array<number>>): number[];
    /**
     * Fit model and returns total distance the centroid has moved.
     * @param {Array<Array<number>>} datas Training data
     * @returns {number} Total distance the centroid has moved
     */
    fit(datas: Array<Array<number>>): number;
}
