/**
 * Complex number
 */
export default class Complex {
    /**
     * @param {number} [real] Real number
     * @param {number} [imag] Imaginary number
     */
    constructor(real?: number, imag?: number);
    /** @private */
    private _real;
    /** @private */
    private _imag;
    /**
     * Real value.
     * @type {number}
     */
    get real(): number;
    /**
     * Imaginary value.
     * @type {number}
     */
    get imaginary(): number;
    /**
     * Returns absolute value.
     * @returns {number} Absolute number
     */
    abs(): number;
    /**
     * Returns conjugate value.
     * @returns {Complex} Conjugate number
     */
    conjugate(): Complex;
    /**
     * Returns added value.
     * @param {number | Complex} other Number to add
     * @returns {Complex} Added complex number
     */
    add(other: number | Complex): Complex;
    /**
     * Returns subtracted value.
     * @param {number | Complex} other Number to subtract
     * @returns {Complex} Subtracted complex number
     */
    sub(other: number | Complex): Complex;
    /**
     * Returns multiplicated value.
     * @param {number | Complex} other Number to multiplicate
     * @returns {Complex} Multiplicated complex number
     */
    mult(other: number | Complex): Complex;
    /**
     * Returns divided value.
     * @param {number | Complex} other Number to divide
     * @returns {Complex} Divided complex number
     */
    div(other: number | Complex): Complex;
    /**
     * Returns sqare root values.
     * @returns {[Complex, Complex]} Sqare root complex numbers
     */
    sqrt(): [Complex, Complex];
    /**
     * Returns cubic root values.
     * @returns {[Complex, Complex, Complex]} Cubic root complex numbers
     */
    cbrt(): [Complex, Complex, Complex];
    /**
     * Returns value of complex exponential function.
     * @returns {Complex} Exponential value
     */
    exp(): Complex;
    /**
     * Returns value of complex log function.
     * @returns {Complex} Principal log value
     */
    log(): Complex;
}
